@@ -1,3 +1,5 @@
+from __future__ import print_function
+
 """SCons.Tool.DCommon
 
 Common code for the various D tools.
@@ -5,8 +7,9 @@ Common code for the various D tools.
 Coded by Russel Winder (russel@winder.org.uk)
 2012-09-06
 """
+
 #
-# Copyright (c) 2001 - 2016 The SCons Foundation
+# Copyright (c) 2001 - 2017 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -28,10 +31,11 @@ Coded by Russel Winder (russel@winder.org.uk)
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "src/engine/SCons/Tool/DCommon.py rel_2.5.1:3735:9dc6cee5c168 2016/11/03 14:02:02 bdbaddog"
+__revision__ = "src/engine/SCons/Tool/DCommon.py rel_3.0.0:4395:8972f6a2f699 2017/09/18 12:59:24 bdbaddog"
 
 import os.path
 
+
 def isD(env, source):
     if not source:
         return 0
@@ -42,6 +46,7 @@ def isD(env, source):
                 return 1
     return 0
 
+
 def addDPATHToEnv(env, executable):
     dPath = env.WhereIs(executable)
     if dPath:
@@ -49,6 +54,14 @@ def addDPATHToEnv(env, executable):
         if os.path.isdir(phobosDir):
             env.Append(DPATH=[phobosDir])
 
+
+def allAtOnceEmitter(target, source, env):
+    if env['DC'] in ('ldc2', 'dmd'):
+        env.SideEffect(str(target[0]) + '.o', target[0])
+        env.Clean(target[0], str(target[0]) + '.o')
+    return target, source
+
+
 # Local Variables:
 # tab-width:4
 # indent-tabs-mode:nil
