@@ -7,9 +7,10 @@ It will usually be imported through the generic SCons.Tool.Tool()
 selection method.
 
 """
+from __future__ import print_function
 
 #
-# Copyright (c) 2001 - 2015 The SCons Foundation
+# Copyright (c) 2001 - 2017 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,7 +32,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "src/engine/SCons/Tool/swig.py rel_2.3.5:3347:d31d5a4e74b6 2015/07/31 14:36:10 bdbaddog"
+__revision__ = "src/engine/SCons/Tool/swig.py rel_3.0.0:4395:8972f6a2f699 2017/09/18 12:59:24 bdbaddog"
 
 import os.path
 import re
@@ -39,9 +40,13 @@ import subprocess
 
 import SCons.Action
 import SCons.Defaults
-import SCons.Scanner
 import SCons.Tool
 import SCons.Util
+import SCons.Node
+
+verbose = False
+
+swigs = [ 'swig', 'swig3.0', 'swig2.0' ]
 
 SwigAction = SCons.Action.Action('$SWIGCOM', '$SWIGCOMSTR')
 
@@ -117,24 +122,33 @@ def _swigEmitter(target, source, env):
             if outdir:
                  java_files = [os.path.join(outdir, j) for j in java_files]
             java_files = list(map(env.fs.File, java_files))
+            def t_from_s(t, p, s, x):
+                return t.dir
+            tsm = SCons.Node._target_from_source_map
+            tkey = len(tsm)
+            tsm[tkey] = t_from_s
             for jf in java_files:
-                t_from_s = lambda t, p, s, x: t.dir
-                SCons.Util.AddMethod(jf, t_from_s, 'target_from_source')
+                jf._func_target_from_source = tkey
             target.extend(java_files)
     return (target, source)
 
-def _get_swig_version(env):
+def _get_swig_version(env, swig):
     """Run the SWIG command line tool to get and return the version number"""
-    pipe = SCons.Action._subproc(env, [env['SWIG'], '-version'],
+    swig = env.subst(swig)
+    pipe = SCons.Action._subproc(env, SCons.Util.CLVar(swig) + ['-version'],
                                  stdin = 'devnull',
                                  stderr = 'devnull',
                                  stdout = subprocess.PIPE)
     if pipe.wait() != 0: return
 
-    out = pipe.stdout.read()
-    match = re.search(r'SWIG Version\s+(\S+)$', out, re.MULTILINE)
+    # MAYBE:   out = SCons.Util.to_str (pipe.stdout.read())
+    out = SCons.Util.to_str(pipe.stdout.read())
+    match = re.search('SWIG Version\s+(\S+).*', out, re.MULTILINE)
     if match:
+        if verbose: print("Version is:%s"%match.group(1))
         return match.group(1)
+    else:
+        if verbose: print("Unable to detect version: [%s]"%out)
 
 def generate(env):
     """Add Builders and construction variables for swig to an Environment."""
@@ -155,8 +169,9 @@ def generate(env):
     java_file.add_action('.i', SwigAction)
     java_file.add_emitter('.i', _swigEmitter)
 
-    env['SWIG']              = 'swig'
-    env['SWIGVERSION']       = _get_swig_version(env)
+    if 'SWIG' not in env:
+        env['SWIG'] = env.Detect(swigs) or swigs[0]
+    env['SWIGVERSION']       = _get_swig_version(env, env['SWIG'])
     env['SWIGFLAGS']         = SCons.Util.CLVar('')
     env['SWIGDIRECTORSUFFIX'] = '_wrap.h'
     env['SWIGCFILESUFFIX']   = '_wrap$CFILESUFFIX'
@@ -168,11 +183,6 @@ def generate(env):
     env['_SWIGINCFLAGS']     = '$( ${_concat(SWIGINCPREFIX, SWIGPATH, SWIGINCSUFFIX, __env__, RDirs, TARGET, SOURCE)} $)'
     env['SWIGCOM']           = '$SWIG -o $TARGET ${_SWIGOUTDIR} ${_SWIGINCFLAGS} $SWIGFLAGS $SOURCES'
 
-    expr = '^[ \t]*%[ \t]*(?:include|import|extern)[ \t]*(<|"?)([^>\s"]+)(?:>|"?)'
-    scanner = SCons.Scanner.ClassicCPP("SWIGScan", ".i", "SWIGPATH", expr)
-
-    env.Append(SCANNERS = scanner)
-
 def exists(env):
     swig = env.get('SWIG') or env.Detect(['swig'])
     return swig
